/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.internal.LoginPopupApplication"); //$NON-NLS-1$

dojo.require("dojo.io.iframe"); //$NON-NLS-1$
dojo.require("jazz.app.auth"); //$NON-NLS-1$
dojo.require("jazz.app.internal.loginUtil"); //$NON-NLS-1$
dojo.require("jazz.app.internal.LoginWidget"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui._Application"); //$NON-NLS-1$

dojo.requireLocalization("jazz.app.internal", "LoginWidgetMessages"); //$NON-NLS-1$

/**
 * This class provides client-side functionality in support of the Jazz server's
 * form-based log in implementation. See <code>net.jazz.web.app/plugin.xml<code>
 * for full documentation.
 */
dojo.declare("jazz.app.internal.LoginPopupApplication", net.jazz.ajax.ui._Application, { //$NON-NLS-1$
	constructor: function() {
		
		// initialize UI
		dojo.body().removeChild(dojo.byId("net-jazz-ajax-InitialLoadMessage")); //$NON-NLS-1$
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "LoginWidgetMessages"); //$NON-NLS-1$
		document.title = this.messages.login;
		
		// determine callback style and initial state
		var query = location.search.substring(1);
		var nvps = query.split("&"); //$NON-NLS-1$
		for(var i=0; i<nvps.length; i++) {
			var nvp = nvps[i].split("="); //$NON-NLS-1$
			if(nvp[0] === "callback" || nvp[0] === "state") { //$NON-NLS-1$ //$NON-NLS-2$
				this[nvp[0]] = nvp[1];
			}
		}
		
		this.iframeIndex = 0;
		
		this.resetUI();
		
		// create initial log in widget
		this.initUI();
	},
	
	resetUI: function() {
		
		this.iframeInitialized = false;
		
		if (this.iframe) {
			dojo.body().removeChild(this.iframe);
			this.iframe = null;
		}
		
		if (this.loginForm) {
			this.loginForm.destroy();
		}
		
		if (this.loginForm) {
			this.dialog.destroy();
		}
	},
	
	initUI: function() {
		
		if (this.callback === "jazz.auth.postSubmit") {
			this.iframe = dojo.io.iframe.create("loginForm" + this.iframeIndex++, "jazz.app.internal.LoginPopupApplication.onload()");
			var self = this;
			jazz.app.internal.LoginPopupApplication.onload = function() {
				//console.warn("initUI onframeload called");
				self.onIFrameLoad();
			}
		}
		
		// create log in form
		this.loginForm = new jazz.app.internal.LoginWidget({
			state: this.state,
			onSubmit: dojo.hitch(this, this.onSubmit)
		});
		
		// create wrapping dialog
		this.dialog = new jazz.ui.Dialog({
			closable: false,
			width: "580px", //$NON-NLS-1$
			trim: jazz.ui.styledBox.TrimStyles.LOGIN,
			contentNode: this.loginForm.domNode
		});
		
		dojo.body().appendChild(this.dialog.domNode);
		
		this.loginForm.startup();
	},
	
	onSubmit: function(userId, password) {
		//console.warn("entered onSubmit");
		
		// if a programmatic client opened this, just call back with the user id
		// and password
		if(this.callback === "jazz.auth.onSubmit") {
			opener[this.callback](userId, password);
			return; // early return
		}
		
		var iframeDoc = dojo.io.iframe.doc(this.iframe);
		
		var form = iframeDoc.createElement("form");
		
		var body = iframeDoc.getElementsByTagName("body")[0];
		
		body.appendChild(form);
		
		form.method = "POST";
		form.action = net.jazz.ajax._contextRoot + "/j_security_check";
		var content = {
			j_username: userId,
			j_password: password
		};
		for(var name in content) {
			var input = iframeDoc.createElement("input");
			input.name = name;
			input.value = content[name];
			form.appendChild(input);
		}
		
		form.submit();
	},
	
	onIFrameLoad: function() {
		
		if(!this.iframeInitialized) {
			
			// do not do anything when the iframe is first created
			this.iframeInitialized = true;
			return;
		}
		
		var iframeUri = new dojo._Url(this.iframe.contentWindow.location);
		
		if(iframeUri.path.indexOf(net.jazz.ajax._contextRoot + "/auth/authfailed") === 0) {
			this.state = "login-failed";
			this.resetUI();
			this.initUI();
		} else {
			//console.warn("sending back uri: " + iframeUri.uri);
			opener[this.callback](iframeUri.uri);
		}
	}
});

})();