/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("net.jazz.ajax.service.ui.internal.userdata.EditFavorite");

dojo.require("net.jazz.ajax.ui.MappingUI");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.Workbench");
dojo.require("net.jazz.ajax.service.ui.userdata.util");
dojo.require("net.jazz.ajax.service.ui.internal.userdata.UserDataEditor");

dojo.require("jazz.client.xhr");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var FAVORITE_EDITOR = "net.jazz.ajax.EditFavorite";

dojo.declare("net.jazz.ajax.service.ui.internal.userdata.EditFavorite", [net.jazz.ajax.service.ui.internal.userdata.UserDataEditor], {
	constructor: function (args) {
		this._dataType = "Favorite";
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		var pathInfo = net.jazz.ajax.ui.MappingUI._getPathInfo();
		var pathParts = pathInfo.split("/");
		var favoritePath = "";
		for (var i=0; i < pathParts.length; i++) {
			if (pathParts[i].indexOf(FAVORITE_EDITOR) >= 0) {
				favoritePath = pathParts.slice(i+1, pathParts.length).join("/"); 
				break;
			}
		}
		
		var dfd = net.jazz.ajax.service.ui.userdata.util.getFavoriteItem(favoritePath);
		if (dfd) {
			dfd.addCallback(dojo.hitch(this, this.processResponse));
			dfd.addCallback(dojo.hitch(this, function(resp) {
				var folder = resp["userdata:folder"].value;
				var dfdFolder = net.jazz.ajax.service.ui.userdata.util.getFavoriteFolderItem(folder);
				dfdFolder.addCallback(dojo.hitch(this, function(folderData) {
					if (folderData["userdata:memberArray"]) {
						for (var i=0; i < folderData["userdata:memberArray"].length; i++) {
							if (folderData["userdata:memberArray"][i].identifier === favoritePath) {
								this.userData.sortKey = folderData["userdata:memberArray"][i].sortKey;
								break;
							}
						}
					} else if (folderData["userdata:member"] && folderData["userdata:member"].identifier === favoritePath) {
						this.userData.sortKey = folderData["userdata:member"].sortKey;
					}
					this.populate();
					return folderData;
				}));
				return resp;
			}));
			dfd.addErrback(dojo.hitch(this, this.favoritesErrback));
		} else {
			this.populate();
		}
		this.connect(this.saveButton, "onclick", "handleSave");
	},
	
	favoritesErrback: function (err) {
		this.processResponseErr(err, "favorites");
	},
	
	handleSave: function (e) {
		e.target.disabled = true;
		var dfd = net.jazz.ajax.service.ui.userdata.util.addFavorite({
			uri: this.userData.uri,
			title: this.titleField.value,
			comments: this.commentsField.value,
			tags: this.tagsField.value,
			folder: this.folderField.value,
			sortKey: this.sortKeyField.value
		});
		dfd.addCallback(dojo.hitch(this, this.saveCallback));
		dfd.addErrback(dojo.hitch(this, this.saveErrback));
	},
	
	saveCallback: function (response) {
		this.userData = {
			uri: this.userData.uri,
			title: this.titleField.value,
			comments: this.commentsField.value,
			tags: this.tagsField.value,
			folder: this.folderField.value,
			sortKey: this.sortKeyField.value
		};
		this.populate();
		return response;
	},
	
	saveErrback: function(err) {
		// TODO - better error message
		alert("Error saving changes: " + dojo.toJson(err));
		return err;
	}
});
})();