/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.SetupUsers");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.validate.regexp");

dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.client.internal.userRegistry");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.LdapRealm");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.SetupOAuth");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("jazz.app.auth");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");


(function() {
	
var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var LicenseAdminClient = com.ibm.team.repository.web.client.internal.licensing;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var userRegistry = com.ibm.team.repository.web.client.internal.userRegistry;
var arrayIndexWithPropertyOfObject = com.ibm.team.repository.web.ui.internal.admin.util.arrayIndexWithPropertyOfObject;
var SetupOAuth = com.ibm.team.repository.web.ui.internal.wizard.SetupOAuth;

var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.wizard.";
var REGISTRY_SERVICE_NAME = "com.ibm.team.repository.service.internal.userregistry.ExternalUserRegistryService";
var REGISTRY_REST_SERVICE_INTERFACE_NAME = "com.ibm.team.repository.service.internal.IExternalUserRegistryRestService"
var REGISTRY_SERVICE_INTERFACE_NAME = "com.ibm.team.repository.common.service.IExternalUserRegistryService";
var REGISTRY_PROPERTY_NAME = "user.registry.type";
var REPO_ITEM_SERVICE_INTERFACE_NAME = "com.ibm.team.repository.service.IRepositoryItemService";
var SERVLET_CONFIG_SERVICE_NAME = "com.ibm.team.repository.servlet.internal.ServletConfigurationService";
var ALLOW_ACCCESS_PROPERTY_NAME = "ws.allow.admin.access";
var SPACE = " ";
var LICENSE_SUPPORTS_LDAP = "com.ibm.team.repository.support.ldap";
var STATUS_TYPES = com.ibm.team.repository.web.ui.wizard.WizardStepHeader.STATUS_TYPES;

var messages;
var tempMessages;

var REGISTRY_TYPE = {
	DETECT: {configValue: "DETECT", display: "JazzRealm"},
	TOMCAT: {configValue: "TOMCAT", display: "JazzRealm"},
	LDAP: {configValue: "LDAP", display: "LdapRealm"},
	UNSUPPORTED: {configValue: "UNSUPPORTED", display: "UnknownRealm"}
}

var COMPUTED_LDAP_PROPS = [
	"findGroupsForUserQuery",
	"findUsersByAnyNameQuery",
	"findUsersByNameQuery",
	"findUsersByUserIdQuery"
]

// The SetupUsers widget provides the basic wizard page implementation. It
// provides three User Registry options to the user:
//
// - Jazz Team Server (really tomcat-users.xml accessed via a Jazz UI)
// - LDAP
// - Non-LDAP External Registry 
//
// Once the user selects one of the user registry options, SetupUsers
// instantiates a form to configure the option the user selected in step 1.
// Each form is defined as a separate "class" at the bottom of this module, and
// all subclass WizardForm (also defined below) to achieve consistent
// behavior.

dojo.declare(PACKAGE_PREFIX + "SetupUsers",  [WizardStep, ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SetupUsers.html"),
	 
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
		tempMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "LdapRealmMessages");
		var JAZZ_ROOT = net.jazz.ajax._contextRoot;
		var adminLicenseLink = "<a href=\'" + JAZZ_ROOT + "/admin#action=com.ibm.team.repository.admin.manageLicenses\'>";
		this.reviewAdminLink = dojo.string.substitute(this.messages.reviewAdminLink, [ adminLicenseLink, "</a>" ]);
		var adminServerLink = "<a href=\"" + JAZZ_ROOT + "/admin#action=com.ibm.team.repository.admin.serverStatus\">";
		this.seeServerStatusLink = dojo.string.substitute(this.messages.seeServerStatus, [adminServerLink, "</a>"]);
		var databaseLink = "<a id='dblink' style='cursor:pointer;'>";
		this.seeDBConfigLink = dojo.string.substitute(this.messages.seeDBConfig, [databaseLink, "</a>"]);
		this.userRegistryDescription = dojo.string.substitute(this.messages.userRegistry, ["<em>", "</em>"]);
		this.stepTitle = messages.wizardPageHeading;
		this.enableNext === false;
		this.evaluateSearchParams();
	},
	
	evaluateSearchParams: function(){
		var qstr = window.location.search.substr(1);
		if (qstr.length) {
			var qparts = qstr.split("&");
			for (var x = 0; x < qparts.length; x++) {
				var tp = qparts[x].split("=");
				if (tp[0] == "ref") {
					this.ref = decodeURIComponent(tp[1]);  // URL to go to upon completion of step 4 instead of Review
				}
				if (tp[0] == "jetty") {
					this.jetty = tp[1];  // flag to test in Jetty environment and leave User Registry setting as DETECT.  Expects "true"/"false".
				}
				if (tp[0] == "oauth") {
					this.oauthName = decodeURIComponent(tp[1]);   // if specified this argument is used as the name for requesting a new approved OAuth key
				}
				if (tp[0] == "oauthId") {
					this.oauthUserId = decodeURIComponent(tp[1]);   // if specified this argument is used as the pre-authorized userId for requesting a new approved OAuth key (requires "oauth" param)
				}
				if (tp[0] == "trust") {
					this.trust = tp[1];   // if specified this argument is used to request an OAuth key as trusted or not trusted.  Expects "true"/"false".
				}
				if (tp[0] == "rootservices") {
					this.rootUrl = decodeURIComponent(tp[1]);  // URL to go to upon completion of step 4 instead of Review
				}
			}
		}
	},

	postCreate: function() {	
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		// When the next button is activated, this checks to see if LDAP queries shoud be 
		// computed.  Then it saves all configuration changes, updates the status, enables the 
		// next button again, and goes to the next page.
		//dojo.connect(this._navigation, "requestNextApproval", this, "_initiateComputeAndSave");
		dojo.connect(this, "_submitConfigCallback", this, "_handleSuccessfulSave"); 
		dojo.connect(this, "_submitConfigError", this, function(errorObj){
			this.setStatus({type: STATUS_TYPES.ERROR, message: messages.errorSavingConfig}, errorObj);
		}); 
		
		// initialize properties
		this.formCache = {};
		this.selectedType = null;
		this._newUserCreated = false;
	}, 
	
	gotoDatabaseStep: function (e) {
		dojo.stopEvent(e);
		this.wizard.selectStep(2);
	},
	
	focusStep: function () {
		if (!this.initialized) {
			this.startLoading();
			this.initialized = true;
		}
	},

	
	startLoading: function() {	
		// set initial status to loading
		this.setStatus({type: "PROGRESS", message: messages.loading});
		disableRegistryButtons(this.registryTypeTable, true);
		
		// do a remote method to get the status of a service to make the LDAP extension service get loaded
		// the registry type radio buttons are disabled until this call returns to ensure the server is ready
		var handler = new SRH(this, "handleServiceStatusResponse", "postCurrentStateLoadError");
		ServerStatusClient.getServiceStatus(handler, REGISTRY_SERVICE_INTERFACE_NAME);	
		
		// check if LDAP is allowed by license 
		var licenseHandler = new SRH(this, "handleLicenseCheck", "postCurrentStateLoadError");
		LicenseAdminClient.checkLicense(licenseHandler, {operationId: LICENSE_SUPPORTS_LDAP} );
	},
	
	handleServiceStatusResponse: function (serviceStatus) {
		// verify that the repo item service is also available before loading the page
		// otherwise the DB is down or has version mismatch
		var handler = new SRH(this, "fetchInitialState", "postCurrentStateLoadError");
		ServerStatusClient.getServiceStatus(handler, REPO_ITEM_SERVICE_INTERFACE_NAME);	
	}, 
	
	fetchInitialState: function (data) {
		if (data.code && data.code === "OK") {
			// shouldn't get here unless the DB status is OK - but contributor may not be in session
			this._noDBAvailable = false;
			var authInSession = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
			if (authInSession) {
				this._defaultAdmin = authInSession == "ADMIN";
				if (this._defaultAdmin) {
					this._loadConfiguration();
				} else {
					// Could be logged-in using LDAP, user would not be ADMIN but doesn't have contributor yet
					this.verifyDefaultAdminBeforeLoad(authInSession);
				}
			} else {
				var handler = new SRH(this, "handleSimpleLoginResponse", "handleSimpleLoginError");
				AdminClient.getSimpleLogin(handler);
			}
		} else {
			this._noDBAvailable = true;
			this.setStatus({type: STATUS_TYPES.ERROR, message: messages.errorLoadingUserRegistry, disableButtons: true});
			// fetch DB status info to let the user know why the services are not available
			ServerStatusClient.getDatabaseStatus(new SRH(this, "displayDatabaseStatusInfo", "postCurrentStateLoadError"));
			dojo.toggleClass(this.licenseLimitedMessage, "noDisplay", this._noDBAvailable);
		}	
		dojo.toggleClass(this.registryTypeStep,"noDisplay", this._noDBAvailable);
		dojo.toggleClass(this.noDatabaseAvailableStep, "noDisplay", !this._noDBAvailable);
		this.createLdapImportInfoBlock();
	},
	
	verifyDefaultAdminBeforeLoad: function (userId) {
		this._fetchItemIdForUserId(userId, "setDefaultAdminState");
	},
	
	setDefaultAdminState: function(contributor) {
		// if there is a contributor for the Logged-in non-admin user, then this is not default admin
		this._defaultAdmin = !(contributor && contributor[0].itemId);
		this._loadConfiguration();
	},
	
	refreshAvailabilityState: function () {
		if (this._noDBAvailable) {
			// check DB and reload
			this.startLoading();
		}
	},
	
	createLdapImportInfoBlock: function () {
		var args = {
			step: "",
			heading: messages.importUserLabel,
			description: messages.importUserDescription,
			messages: messages
		};
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		var ldapInfo = new constructor(args);		
		this.ldapImportInfoBlock.appendChild(ldapInfo.domNode);
	},
	
	handleSimpleLoginResponse: function (data) {
		this._defaultAdmin = data && data === "ADMIN";
		this._loadConfiguration();
	},
	
	handleSimpleLoginError: function (err) {
		this.setStatus({type: STATUS_TYPES.ERROR, message: messages.accessUserError, disableButtons: true}, err);		
	} ,
	
	_loadConfiguration: function () {
		// get the current setting for user registry type
		var handler = new SRH(this, "postCurrentStateLoad", "postCurrentStateLoadError");
		ServerStatusClient.getServiceConfiguration(handler, REGISTRY_SERVICE_NAME);
		
		// get the current setting for allowing admin access
		var handler = new SRH(this, "postServletConfigurationLoad", "postCurrentStateLoadError");
		ServerStatusClient.getServiceConfiguration(handler, SERVLET_CONFIG_SERVICE_NAME);	
		
		// fetch the CALs
		if (this._defaultAdmin && !this._newUserCreated) {
			this.getLicensingInfo();
		}
	},
	
	displayDatabaseStatusInfo: function(dbStatus) {
		dojox.data.dom.removeChildren(this.noDatabaseAvailableStep);
		if (dbStatus.code && dbStatus.code === STATUS_TYPES.ERROR) {
			statmessage = messages.dbUnavailable+ "<br>" + dbStatus.message;			
		} else {
			statmessage = messages.unknownError + SPACE + SPACE + this.seeServerStatusLink;
		}
		this.setStatus({type: STATUS_TYPES.ERROR, message: statmessage, disableButtons: true});
		var args = {
			step: "",
			heading: messages.configureDbFirstHeading,
			description: messages.configureDbFirstDescription + SPACE + SPACE + this.seeDBConfigLink,
			messages: messages
		};
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		var dbInfo = new constructor(args);
		this.noDatabaseAvailableStep.appendChild(dbInfo.domNode);	
		
		var dbElemLink = dojo.byId("dblink");
		dojo.connect(dbElemLink,"onclick",this,"gotoDatabaseStep");
	},
	
	handleLicenseCheck: function(response){
		this.allowsLdap = response.code && response.code === "OK";
		// show LDAP as option if allowed by license or message if not
		dojo.toggleClass(this.registryTableLdapRow, "noDisplay", !this.allowsLdap);	
		dojo.toggleClass(this.licenseLimitedMessage, "noDisplay", this.allowsLdap);
	},
	
	postCurrentStateLoad: function(currentStateInfo) {
		// Display the current setting for user registry type.  Options:  LDAP, TOMCAT, DETECT, UNSUPPORTED		
		var index = arrayIndexWithPropertyOfObject(currentStateInfo.properties, "name", REGISTRY_PROPERTY_NAME);
		var registryType = currentStateInfo.properties[index];
		this.savedRegistryType = registryType.overrideValue;
		var registryTypeValue = registryType.overrideValue != null ? registryType.overrideValue : registryType.defaultValue;
				
		if (registryTypeValue === REGISTRY_TYPE.DETECT.configValue) { 
			userRegistry.getUserRegistryType(new SRH(this, "postDetectRegistryType", "postCurrentStateLoadError"));
		} else {
			this._setupRegistryType(registryTypeValue);
		}	
		disableRegistryButtons(this.registryTypeTable, false);	
	},
	
	postServletConfigurationLoad: function(servletConfigInfo) {
		// if ADMIN access is not disabled, show checkbox to disable it
		var index = arrayIndexWithPropertyOfObject(servletConfigInfo.properties, "name", ALLOW_ACCCESS_PROPERTY_NAME);
		var allowAdmin = servletConfigInfo.properties[index];
		var allowAdminValue = allowAdmin.overrideValue != null ? allowAdmin.overrideValue : allowAdmin.defaultValue;
				
		if (allowAdminValue === "true") { 
			dojox.data.dom.removeChildren(this._disableAdminArea);
			var content = document.createElement("div"); 
			content.appendChild(document.createTextNode(messages.disableAdmin));
			var allowAdminForm = document.createElement("div"); 
			allowAdminForm.className = "spaced-bottom-check";
			this.allowAdminCheck = WizardForm.createInputElement({type: "checkbox", name: "disableAdmin"}); 
			allowAdminForm.appendChild(this.allowAdminCheck);
			this.allowAdminCheck.checked = true;
			this.connect(this.allowAdminCheck,"onclick","handleAdminCheckChange");
			var allowAdminLabel = document.createElement("label");
			allowAdminLabel.htmlFor = "disableAdmin"
			allowAdminLabel.innerHTML = messages.disableAdminLabel;
			allowAdminLabel.className ="spaced";
			allowAdminForm.appendChild(allowAdminLabel);
			content.appendChild(allowAdminForm);
			var args = {
				step: messages.step3Label,
				heading: messages.disableAdminLabel,
				description: content, 
				messages: messages
			};
			var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
			this.disableAdminForm = new constructor(args);
			this._disableAdminArea.appendChild(this.disableAdminForm.domNode);	
			
			this.adminPasswordInfoNode = document.createElement("div");
			this.adminPasswordInfoNode.className = "spaced-all warning-text noDisplay";
			this.adminPasswordInfoNode.innerHTML = messages.changeAdminPasword;
			this.allowAdminCheck.parentNode.appendChild(this.adminPasswordInfoNode);
			
		} else {
			this._skipStep3 = true;
		} 
	},
	
	handleAdminCheckChange: function () {
		dojo.toggleClass(this.adminPasswordInfoNode, "noDisplay", this.allowAdminCheck.checked);
	},
	
	_setupRegistryType: function(type) {
		setSelectedRegistryButton(this.registryTypeTable, type);
		this.displaySelectedRegistryType(type);
	}, 
	
	postDetectRegistryType: function (type) {
		//TODO ignore case in case of Tomcat
		if (type === "tomcat") {
			targetType = REGISTRY_TYPE.TOMCAT.configValue;
		} else if (this.allowsLdap){
			targetType = REGISTRY_TYPE.LDAP.configValue;
		} else {
			targetType = REGISTRY_TYPE.UNSUPPORTED.configValue;
		}
		this._setupRegistryType(targetType);
	},
	
	postCurrentStateLoadError: function(errorObj) {
		this.setStatus({type: STATUS_TYPES.ERROR, message: messages.errorLoadingUserRegistry, disableButtons: true}, errorObj);
	},
	
	onSelectRegistryType: function(e) {
		if(!e.target || e.target.tagName !== "INPUT") {return;}
		
		var selected = getSelectedRegistryType(this.registryTypeTable);
		this.displaySelectedRegistryType(selected);
	},
	
	displaySelectedRegistryType: function(type) {
		this.formContainer.className = "net-jazz-ajax-block";
		this.selectedType = type;
		var formType = REGISTRY_TYPE[type].display;
		if (!this._defaultAdmin || this._newUserCreated) {
			this.hideClientLicenseForm();
			if (type != REGISTRY_TYPE.LDAP.configValue) {
				formType = "NoNewUserRequired";
			}
		}
		var showLdapImport = type === REGISTRY_TYPE.LDAP.configValue && this._defaultAdmin && !this._newUserCreated;
		dojo.toggleClass(this.ldapImportInfoBlock,"noDisplay", !showLdapImport);

		var widget = this.formCache[formType];
		if(!widget) {
			var constructor = dojo.getObject(PACKAGE_PREFIX + formType);
			var args = {
				step: messages.step2Label,
				setStatusCallback: dojo.hitch(this, this.setStatus)
			};
			widget = new constructor(args);
			this.formCache[formType] = widget;
		}
		this.currentForm = widget;
		if (widget.validate != null) {
			widget.validate();
		}
		
		if(this.formContainer.firstChild){
			if(this.formContainer.firstChild !== widget.domNode) {
				this.formContainer.replaceChild(widget.domNode, this.formContainer.firstChild);
			}
		} else{
			this.formContainer.appendChild(widget.domNode)
		}
	},
	
	hideClientLicenseForm: function () {
		dojox.data.dom.removeChildren(this._licenseAssignmentList);
		this.licenseForm = null;
	}, 
	
	setStatus: function(status, errorObj,hideBottomStatus) {
		var canClickNext = false;
		if (status.type === STATUS_TYPES.OK || (status.overrideNext != null && status.overrideNext)) {
			canClickNext = true;
		}
		this.enableNext = canClickNext;
		if (this.wizard) {
			this.wizard.wizardNextButton.disabled = true;
			this.wizard.updateNavigationControls();
			// added to override the Next button even if there is not a next page for WI 86253
			if(this.ref && this.canFlipToNextPage()) {
				this.wizard.wizardNextButton.disabled = false;
			}
		}
		if (!status.disableButtons) {
			disableRegistryButtons(this.registryTypeTable, false);
		}
		if(canClickNext && status.type === STATUS_TYPES.OK) {
			status.message = messages.success + messages.clickNext;
		}
		this.displayStatus(status, errorObj,hideBottomStatus);
	}, 
	
	displayStatus: function(status, errorObj,hideBottomStatus) {
		var details = errorObj ? errorObj :status.errorObj;
		// only send ERROR type to error callback path if from error callback
		if (status.type === STATUS_TYPES.ERROR && (errorObj || status.fromErrorCallback)) {
			this.header.displayServiceCallbackErrorMessage(details, status.message);
		}
		else {
			this.header.showStatus({type: status.type,	summary: status.message, details: details})
		}
		if(hideBottomStatus) {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
		} else {
			this._setStatusIndication(status.type, status.message);
		}
	},
	
	_setStatusIndication: function (type, summary) {
		if (type != "INFO" && type != "NONE") {
			var indication = document.createElement("div");
			indication.className = "stepStatus statusMsg " + type;
			indication.innerHTML = summary;

			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(indication);
			if (type != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}
		} else {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
		}
	}, 
	
	// Licensing 
	getLicensingInfo: function () {
		LicenseAdminClient.getLicenseInfo(new SRH(this, "getLicensingInfoCallback", "handleLicensingInfoError"));
	},
			
	getLicensingInfoCallback: function (data) {
		dojox.data.dom.removeChildren(this._licenseAssignmentList);
		var args = {
			step: this._skipStep3 ? messages.step3Label : messages.step4Label,
			heading: messages.assignLicenseHeading,
			description: messages.selectClientLicense
		};
		var enabled = false;
		if (data.types != null) {
			var licenses = LicenseAdminClient.sortCALs(data.types);
			licenses = filterOutFunctionalCALs(licenses);
			var licenseRowData = [];
			for (var i=0; i < licenses.length; i++) {
				var CALInfo = LicenseAdminClient.getClientAccessLicenseInfo(data, licenses[i])
				var licTypeNode = document.createElement("div");
				licTypeNode.innerHTML = "<label for='" + licenses[i].id + "'>" + CALInfo.name + "</label>";
				var descriptionNode = document.createElement("div");
				descriptionNode.className = "inline-description";
				descriptionNode.innerHTML = licenses[i].description;
				licTypeNode.appendChild(descriptionNode);
				
				var disableCAL = !CALInfo.assignableToUserWithoutCALs;
				licenseRowData.push({
					name: licenses[i].id, 
					type: "checkbox", 
					label: licTypeNode, 
					description: licenses[i].description,
					availability: CALInfo.simpleAvailabilityString,
					disabled: disableCAL
				});		
				if (!disableCAL){
					enabled = true;
				}
			}
			if (!LicenseAdminClient.isServerAcceptingNewLicensedUsers(data)) {	
				var descNode = document.createElement("div");
				var descTextNode = document.createElement("div");
				descTextNode.innerHTML = enabled? args.description: messages.noLicensesEnabled;
				descNode.appendChild(descTextNode);
				var maxNode = this.createWarningNode(LicenseAdminClient.SERVER_COUNT_MAX_MSG + this.reviewAdminLink);
				descNode.appendChild(maxNode);
				args.description = descNode;
			} else if (!enabled){
				args.description = this.createWarningNode(messages.noLicensesEnabled + this.reviewAdminLink);
			}
			args.tableContent = { header: CLIENT_LICENSE_HEADER_DEF, rows: licenseRowData};	
		}
		else {
			args.description = this.createWarningNode(messages.noClientLicenses + SPACE + SPACE + this.reviewAdminLink);
		}
		args.messages = messages;
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		this.licenseForm = new constructor(args);
		this._licenseAssignmentList.appendChild(this.licenseForm.domNode);	
	},

	createWarningNode: function(message) {
		var warningNode = document.createElement("div");
		warningNode.className = "WARNING spaced-all";
		warningNode.innerHTML = message;
		return warningNode;
	}, 
	
	handleLicensingInfoError: function(errorObj) {
		this.setStatus({type: STATUS_TYPES.ERROR, message: messages.errorFetchingLicense}, errorObj);
		var constructor = dojo.getObject(PACKAGE_PREFIX + "StepBox");
		var args = {
			step: this._skipStep3 ? messages.step3Label : messages.step4Label,
			heading: messages.assignLicenseHeading,
			description: messages.errorFetchingLicense + SPACE + SPACE + messages.seeLog, 
			messages: messages
		};
		this.licenseForm = new constructor(args);
		this._licenseAssignmentList.appendChild(this.licenseForm.domNode);	
	},
	
	// handle Users
	_fetchItemIdForUserId: function(user, successPath) {
		var args = {
			userId: user
		}
		RepositoryClient.getContributors(new SRH(this, successPath, "postContributorSearchError"), args);
	}, 
	
	postContributorSearchResult: function(contributor) {
		if (contributor && contributor[0].itemId) {
			var licenseChanges = this._getLicenseChanges();
			this.currentForm.runAction(contributor[0].itemId, contributor[0].stateId, licenseChanges, this);
		} else {
			this.setStatus({type: STATUS_TYPES.WARNING, message: messages.loginLdapImport});
		}	
	}, 
	
	postContributorSearchError: function(errorObj) {
		this.setStatus({type: STATUS_TYPES.ERROR, message: messages.newUserLookupError}, errorObj);
	},
	
	// SUPER ConfigUtility METHODS	
	_getChanges: function () {
		var changes = [];
		var properties = [];
		var registryType = this.selectedType != null ? this.selectedType : REGISTRY_TYPE.DETECT.configValue;
		if (registryType === REGISTRY_TYPE.LDAP.configValue && this.currentForm.ldapConfig._changed) {
			var changeObj = this.currentForm.ldapConfig._getChanges();
			changes = changeObj["changes"];
			properties = changeObj["properties"];
		}
		if (this.savedRegistryType != registryType) {
			var registryChange = {
				name: REGISTRY_PROPERTY_NAME,
				value: registryType,
				serviceImplementationName: REGISTRY_SERVICE_NAME
			};
			changes.push(registryChange);
		}
		return {
			"changes": changes,
			"properties": properties
		};

	},
	
	// WIZARD STEP
	canFlipToNextPage: function () {
		return this.enableNext === true;
	},
	
	onNext: function() {
		this._initiateComputeAndSave();
	},
	
	// SAVE METHODS
	_initiateComputeAndSave: function() {
		this.enableNext = false;
		if (this.wizard) {
			this.wizard.updateNavigationControls();
		}
		this.setStatus({type: "PROGRESS", message: messages.savingSettings});
		if (this.selectedType == null) {
			this._handleInvalidRegistrySelection();
			return;
		}
		disableRegistryButtons(this.registryTypeTable, true);
		if (this.selectedType === REGISTRY_TYPE.LDAP.configValue) {
			this._computeLdapQueries();
		} else {
			this._validateNonLdapRegistry();
		}	
	}, 

	_validateNonLdapRegistry: function() {
		var status = this.currentForm.validate();
		
		// JUST FOR TESTING LOGIN ON JETTY
		if (this.jetty && this.jetty == "true") {
			this.selectedType = "DETECT";
		}

		if(status.isValid && status.isComplete){
			this._invokeSaveAfterCompute();
		} else {
			this.setStatus(status);
		}
	},
	
	_computeLdapQueries: function() {
		if (this.currentForm.computeLdapOption.checked) {			
			var pendingUpdates = this.currentForm.ldapConfig._getAllValuesExceptSensitiveReplacements();
			if (pendingUpdates.length === 0) {
				return false;
			}
			var properties = {};
			for (var i=0; i<pendingUpdates.length; i++) {
				var name = pendingUpdates[i].name;
				var value = pendingUpdates[i].value;
				properties[name] = value;
			};
			
			var handler = new SRH(this, "cacheComputedQueries", "postComputeQueryError");
			userRegistry.computeLDAPQueries(handler, {basicProperties: dojo.toJson(properties)});
		} else {
			this._invokeSaveAfterCompute();
		}
	},
	
	cacheComputedQueries: function (results) {	
		var ldapProperties = this.currentForm.ldapConfig;	
		for (var i=0; i<COMPUTED_LDAP_PROPS.length; i++) {
			// find the prop in ldap props with matching name.  set it's value and changed flag
			var index = arrayIndexWithPropertyOfObject(ldapProperties._properties, "_name", "ldap." + COMPUTED_LDAP_PROPS[i]);
			var propertyElement = ldapProperties._properties[index];
			if (propertyElement._editValueNode.value != results[COMPUTED_LDAP_PROPS[i]]) {
				propertyElement._editValueNode.value = results[COMPUTED_LDAP_PROPS[i]];
				propertyElement._changed = true;
				ldapProperties._changed = true;
				propertyElement._validationPending = false;
			}
		};
		
		this._invokeSaveAfterCompute();
	}, 
	
	postComputeQueryError: function(errorObj) {
		this.setStatus({type: STATUS_TYPES.ERROR, message: messages.computeQueryError}, errorObj);
	},
	
	_invokeSaveAfterCompute: function () {
		// If any configuration properties need to be saved, _handleSuccessfulSave will be called
		// when the updates are complete.  Otherwise, proceed to create the new user
		var invokedSave = this._saveHandler();
		if (!invokedSave) {
			this._initiateCreateUser();
		}
	}, 
	
	_handleSuccessfulSave: function() {
		this.savedRegistryType = this.selectedType != null ? this.selectedType : REGISTRY_TYPE.DETECT.configValue;
		this._initiateCreateUser();	
	},
	
	_initiateCreateUser: function () {
		if (!this._defaultAdmin || this._newUserCreated){
			this.disableAdminAccess();
			return;
		}
		if (this.savedRegistryType === REGISTRY_TYPE.LDAP.configValue) {
				userRegistry.importUserFromRegistry(new SRH(this, "postCreateCallback", "postImportUserError"));
		} else {
			var valueMap = this.currentForm.getData();
			var licenseChanges = this._getLicenseChanges();
			this.currentForm.runAction(valueMap, licenseChanges, this);			
		}
	},
	
	_getLicenseChanges: function() {
		var licenseChanges = {
			add: [],
			remove: []
		};
		if (this.licenseForm && this.licenseForm.getData()) {
			var assigned = this.licenseForm.getData();
			for (i in assigned) {
				licenseChanges[(assigned[i] ? "add" : "remove")].push(i);
			}
		}
		return licenseChanges;
	},
	
	postCreateCallback: function(data) {
		this._newUserCreated = true;
		if (this.savedRegistryType === REGISTRY_TYPE.TOMCAT.configValue || this.savedRegistryType === "DETECT") {
			var newUserId = this.currentForm.getData().userId;
			var newPassword = this.currentForm.getData().password;
			var loginDfd = jazz.app.auth.login({userId: newUserId, password: newPassword});

			loginDfd.addCallback(dojo.hitch(this, this.verifyNewLogin));
			loginDfd.addErrback(dojo.hitch(this, this.postLoginErrorCallback));
		} else if (this.savedRegistryType === REGISTRY_TYPE.LDAP.configValue) { 
			if (data){
				this._fetchItemIdForUserId(data, "postContributorSearchResult");
			} else {
				this.setStatus({type: STATUS_TYPES.WARNING, message: messages.loginLdapImport});
			}
		} else {
			this.disableAdminAccess();
		}		
	},
	
	verifyNewLogin: function () {
		// call this after switching to new user
		var handler = new SRH(this, "updateGlobalLogin", "handleSimpleLoginError");
		AdminClient.getSimpleLogin(handler);
	}, 
	
	updateGlobalLogin: function (data) {
		// update global user var to be new logged-in user since it may be used by other wizard steps
		dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", data);
		
		//update local default ADMIN
		this._defaultAdmin = data && data === "ADMIN";
		
		this.disableAdminAccess();
	},
	
	disableAdminAccess: function () {
		if (this.allowAdminCheck != null) {
			this.checkLoginDisableAdminResponse();
		} else {
			this.postFinishSetupUsers();
		}
	},
	
	checkLoginDisableAdminResponse: function (data) {
		// check AUTHENTICATED_USERID instead of defaultAdmin here for new imported LDAP
		var authInSession = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		if (authInSession && authInSession != "ADMIN") {
			var newValue = !this.allowAdminCheck.checked ? "true" : "false";
			var propUdpate = [{
				name: ALLOW_ACCCESS_PROPERTY_NAME,
	 			value: newValue,
	 			serviceImplementationName: SERVLET_CONFIG_SERVICE_NAME     
			}];
			var changeArgs = {
				updates: dojo.toJson(propUdpate)
			};
			var srh = new SRH(this, "postRemoveAdminRoles", "postAdminAccessError");
			ServerStatusClient.postConfigurationUpdates(srh, changeArgs);
		} else {
			if (this.allowAdminCheck.checked) {
				// should be new user logged in, but still ADMIN
				this.setStatus({type: STATUS_TYPES.WARNING, message: messages.adminNotDisabled + messages.clickNext, overrideNext: true});
				this.allowAdminCheck.checked = false;
				this.handleAdminCheckChange();
				this.allowAdminCheck.disabled = true;
				
				var descNode = document.createElement("div");
				descNode.className = "WARNING spaced-all";
				descNode.innerHTML = messages.adminNotDisabled;
				this.allowAdminCheck.parentNode.insertBefore(descNode, this.allowAdminCheck);
			} else {
				this.postFinishSetupUsers();
			}
		} 
	},
	
	postRemoveAdminRoles: function () {
		// check AUTHENTICATED_USERID instead of defaultAdmin here for new imported LDAP
		var authInSession = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID");
		if (authInSession && authInSession != "ADMIN") {
			if (this.allowAdminCheck.checked){
				// if allow admin access was disabled, also remove the roles for ADMIN
				AdminClient.postRemoveAdminRoles(new SRH(this, "postFinishSetupUsers", "postAdminRolesError"));
			} else {
				// admin access wasn't disabled, so continue on
				this.postFinishSetupUsers();
			}
		} else {
			// shouldn't be able to get here as ADMIN user
			this.setStatus({type: STATUS_TYPES.WARNING, message: messages.adminRolesNotDisabled + messages.clickNext, overrideNext: true});
		}
	},
	
	postLoginErrorCallback: function (err) {
		this.setStatus({type: STATUS_TYPES.WARNING, message: messages.newUserLoginError, overrideNext: true}, err);
	},
	
	postCreateErrorCallback: function (err) {
		if (err.message != null) {
			this.setStatus({type: STATUS_TYPES.ERROR, message: messages.newUserCreateError}, err);
		} else {
			this.setStatus({type: STATUS_TYPES.WARNING, message: messages.newUserCreateUnknownError, overrideNext: true}, err);
		}
	},
	
	postFinishSetupUsers: function() {
		this.displaySelectedRegistryType(this.savedRegistryType);
		if (this.oauthName) {
			this.createOAuthConsumer();
		} else {
			this.updateFinishedNavigation();
		}
	},
	
	updateFinishedNavigation: function () {
		this.setStatus({type: STATUS_TYPES.OK});
		if (!this.ref) {
			this.wizard.processNext();
		} else {
			location.href=this.ref;
		}
	},
	
	createOAuthConsumer: function() {
		var setupOAuth = new SetupOAuth();
		var args = {
			consumerName: this.oauthName,
			trust: this.trust, 
			userId: this.oauthUserId,
			rootUrl: this.rootUrl
		}	
		setupOAuth.initParams(this, "handleCreateOAuthSuccess", "handleCreateOAuthError", args);	
	},
	
	handleCreateOAuthSuccess: function(result){
		if (this.ref) {
			var refParts = this.ref.split("#");
			// ref without hash
			var updatedRef = refParts[0];
			
			var oauthParams = "";
			if(result.consumerKey) {
				oauthParams += "k=" + encodeURIComponent(result.consumerKey);
			}
			if(result.consumerSecret) {
				oauthParams += "&s=" + encodeURIComponent(result.consumerSecret);
			}
			
			var paramIndex = updatedRef.indexOf("?");
			if (paramIndex != -1) {
				updatedRef = updatedRef + "&" + oauthParams;
			}
			else {
				updatedRef = updatedRef + "?" + oauthParams;
			}
			if (refParts.length > 1) {
				// add hash back on
				updatedRef = updatedRef + "#" + refParts[1];
			}			
			this.ref = updatedRef;
			this.setStatus({type: STATUS_TYPES.OK});
			location.href=this.ref;
		}
	},
	
	handleCreateOAuthError: function(errorObj) {
		this.setStatus({type: STATUS_TYPES.ERROR, message: messages.createOAuthError}, errorObj);
	},
	
	postAssignLicensesError:  function(error) {
		this.setStatus({type: STATUS_TYPES.WARNING, 
			message: messages.licenseAssignError, 
			overrideNext: true}, error);
	},
	
	postAdminAccessError:  function(error) {
		this.setStatus({type: STATUS_TYPES.WARNING, 
			message: messages.disableAdminError, 
			overrideNext: true}, error);
	},	
	
	postAdminRolesError:  function(error) {
		this.setStatus({type: STATUS_TYPES.WARNING, 
			message: messages.removeRolesError, 
			overrideNext: true}, error);
	},	
	
	postImportUserError: function(error) {
		this.setStatus({type: STATUS_TYPES.WARNING, 
			message: messages.newUserCreateError, 
			overrideNext: true}, error);
	},
	
	_handleInvalidRegistrySelection: function() {
		this.setStatus({type: STATUS_TYPES.INFO, message: messages.selectRegistry});
	}
	
});

var CREATE_USER_HEADER_DEF = [
	{type: "label", labelKey: "propertyLabel"},
	{type: "data", labelKey: "valueLabel"},
	{type: "description", labelKey: "descriptionLabel"}
];

var USER_ID_FIELD = {name: "userId", labelKey: "userIdLabel", descriptionKey: "userIdDescrption"};
var NAME_FIELD = {name: "name", labelKey: "nameLabel", descriptionKey: "nameDescrption"};
var EMAIL_ADDRESS_FIELD = {name: "emailAddress", labelKey: "emailLabel", dataType: "email", descriptionKey: "emailDescrption"};

var JAZZ_REALM_TABLE = {
	header: CREATE_USER_HEADER_DEF,
	rows: [
		USER_ID_FIELD,
		NAME_FIELD,
		{name: "password", labelKey: "passwordLabel", type: "password", descriptionKey: "passwordDescrption"},
		{name: "repeatPassword", labelKey: "passwordLabel2", type: "password", descriptionKey: "passwordDescrption2", mustMatch: {
			matchName: "password",
			errorMessageKey: "passwordMismatch"
		}},
		EMAIL_ADDRESS_FIELD
	]
};

var UNKNOWN_REALM_TABLE = {
	header: CREATE_USER_HEADER_DEF,
	rows: [
		USER_ID_FIELD,
		NAME_FIELD,
		EMAIL_ADDRESS_FIELD
	]
};

var CLIENT_LICENSE_HEADER_DEF = [
	{type: "data", label: " ", className: "slight-tight-cell"},
	{type: "label", labelKey: "typeLabel"},
	{type: "availability", labelKey: "availabilityLabel", className: "wizardTypeCell"}
];

function getSelectedRegistryType(registryTypeTable) {
	var inputs = registryTypeTable.getElementsByTagName("input");
	var value = null;
	for (var i=0; i<inputs.length; i++) {
		if (inputs[i].checked) {
			value = inputs[i].value;
			break;
		}
	}
	return value;
}

function setSelectedRegistryButton(registryTypeTable, target) {
	var inputs = registryTypeTable.getElementsByTagName("input");
	for (var i=0; i<inputs.length; i++) {
		if (inputs[i].value === target) {
			inputs[i].checked = true;
			break;
		}
	}
}

function disableRegistryButtons(registryTypeTable, disable) {
	var inputs = registryTypeTable.getElementsByTagName("input");
	for (var i=0; i<inputs.length; i++) {
		inputs[i].disabled = disable;
	}
}

function filterOutFunctionalCALs(calList){
	var filteredCALs = [];
	dojo.forEach(calList, function(cal){
		if (!cal.functional) {
			filteredCALs.push(cal);
		}
	});
	return filteredCALs;
}	


// The following two classes provide implementations for the different user
// registry types (Jazz Team Server, Non-LDAP External Registry). They
// subclass WizardForm for common validation logic, and WizardForm
// calls down to these classes' runAction method when the user presses the
// form's action button (e.g. "Create User", "Test connection").

dojo.declare(PACKAGE_PREFIX + REGISTRY_TYPE.TOMCAT.display, WizardForm, {
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
		this.heading = messages.createAccountLabel;
		this.description = messages.createAccountDescription;
	}, 
	
	tableContent: JAZZ_REALM_TABLE,
		
	runAction: function(valueMap, licenses, caller) {
		// alert("Run " + REGISTRY_TYPE.TOMCAT.display + " action {" + objToStr(valueMap) + "}");
		var args = new Object();
		for (i in valueMap) {
			if (i == "repeatPassword") {
				continue;
			}
			args[i] = valueMap[i];
		}
		args["itemId"] = "new";
		args["jsonRoles"] = dojo.toJson(["JazzAdmins"]);
		args["jsonLicenses"] = dojo.toJson(licenses);

		AdminClient.postContributor(args, new SRH(caller, "postCreateCallback", "postCreateErrorCallback"));
	},
	
	validate: function() {
		var status = this.inherited("validate", arguments);
		if (!status.isValid) {
			status.message = tempMessages.pendingErr;
		} else {
			status.message = messages.createUser;
		}
		this.setStatusCallback(status);
		return status;
	}
});

dojo.declare(PACKAGE_PREFIX + REGISTRY_TYPE.UNSUPPORTED.display, WizardForm, {
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
		this.heading =  messages.createUserLabel;
		this.description = messages.createUserDescription;
	},
	
	tableContent: UNKNOWN_REALM_TABLE,
		
	runAction: function(valueMap, licenses, caller) {
		//alert("Run " + REGISTRY_TYPE.UNSUPPORTED.display + " action {" + objToStr(valueMap) + "}");
		var args = new Object();
		for (i in valueMap) {
			args[i] = valueMap[i];
		}
		args["itemId"] = "new";
		args["repoOnly"] = true; 
		args["jsonLicenses"] = dojo.toJson(licenses);

		AdminClient.postContributor(args, new SRH(caller, "postCreateCallback", "postCreateErrorCallback"));
	},
	
	validate: function() {
		var status = this.inherited("validate", arguments);
		if (!status.isValid) {
			status.message = tempMessages.pendingErr;
		} else {
			status.message = messages.createUser;
		}
		this.setStatusCallback(status);
		return status;
	}
});

dojo.declare(PACKAGE_PREFIX + "NoNewUserRequired", WizardForm, {
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupUsersMessages");
		this.heading =  messages.userExistsLabel;
		this.description = messages.userExistsDescrpition;
	},
	
	validate: function() {
		var status  = {
			type: STATUS_TYPES.OK,
			isComplete: true,
			isValid: true
		};
		this.setStatusCallback(status);
		return status;
	}, 
	
	runAction: function(valueMap, licenses, caller) {
		caller.disableAdminAccess();
	}	
});

dojo.declare(PACKAGE_PREFIX + "StepBox", WizardForm, {
});


})();